package controllers

import (
	"encoding/json"
	"eta/eta_mini_api/models"
	"eta/eta_mini_api/services"
	"eta/eta_mini_api/utils"
	"net/http"
	"net/url"

	"github.com/beego/beego/v2/server/web"
)

type BaseCommonController struct {
	web.Controller
}

func (c *BaseCommonController) Prepare() {
	var requestBody string
	method := c.Ctx.Input.Method()
	if method == "GET" {
		requestBody = c.Ctx.Request.RequestURI
	} else {
		requestBody, _ = url.QueryUnescape(string(c.Ctx.Input.RequestBody))
	}

	ip := c.Ctx.Input.IP()
	utils.ApiLog.Info("uri:%s, requestBody:%s, ip:%s", c.Ctx.Input.URI(), requestBody, ip)
}

func (c *BaseCommonController) ServeJSON(encoding ...bool) {
	var (
		hasIndent   = false
		hasEncoding = false
	)
	if web.BConfig.RunMode == web.PROD {
		hasIndent = false
	}
	if len(encoding) > 0 && encoding[0] == true {
		hasEncoding = true
	}
	if c.Data["json"] == nil {
		go utils.SendEmail(utils.APPNAME+" "+utils.RunMode+"异常提醒", "接口:"+"URI:"+c.Ctx.Input.URI()+";无返回值", utils.EmailSendToUsers)
		return
	}
	baseRes := c.Data["json"].(*models.BaseResponse)
	if baseRes != nil && !baseRes.Success && baseRes.IsSendEmail {
		go utils.SendEmail(utils.APPNAME+" "+utils.RunMode+" 失败提醒", "URI:"+c.Ctx.Input.URI()+" ErrMsg:"+baseRes.ErrMsg+";Msg"+baseRes.Msg, utils.EmailSendToUsers)
	}
	c.JSON(c.Data["json"], hasIndent, hasEncoding)
}

func (c *BaseCommonController) JSON(data interface{}, hasIndent bool, coding bool) error {
	c.Ctx.Output.Header("Content-Type", "application/json; charset=utf-8")
	var content []byte
	var err error
	if hasIndent {
		content, err = json.MarshalIndent(data, "", "  ")
	} else {
		content, err = json.Marshal(data)
	}
	if err != nil {
		http.Error(c.Ctx.Output.Context.ResponseWriter, err.Error(), http.StatusInternalServerError)
		return err
	}
	ip := c.Ctx.Input.IP()
	requestBody, _ := url.QueryUnescape(string(c.Ctx.Input.RequestBody))
	utils.ApiLog.Info("请求地址：", c.Ctx.Input.URI(), "Authorization:", c.Ctx.Input.Header("Authorization"), "RequestBody：", requestBody, "ResponseBody", string(content), "IP：", ip)

	if coding {
		content = []byte(utils.StringsToJSON(string(content)))
	}

	// 数据加密
	if services.CheckEncryption() {
		content = utils.DesBase64Encrypt(content, utils.Key)
		// get请求时，不加双引号就获取不到数据，不知道什么原因，所以还是在前后加上双引号吧
		content = []byte(`"` + string(content) + `"`)
	}

	return c.Ctx.Output.Body(content)
}
