package wechat

import (
	"eta/eta_mini_api/models"
	"eta/eta_mini_api/utils"
	"fmt"
	"time"

	"github.com/silenceper/wechat/v2"
	"github.com/silenceper/wechat/v2/cache"
	"github.com/silenceper/wechat/v2/credential"
	"github.com/silenceper/wechat/v2/officialaccount"
	"github.com/silenceper/wechat/v2/officialaccount/config"
	"github.com/silenceper/wechat/v2/officialaccount/js"
	"github.com/silenceper/wechat/v2/officialaccount/user"
)

var (
	WxAppId     string
	WxAppSecret string
)

type WechatAccessToken struct {
}

func GetWxChat() (officialAccount *officialaccount.OfficialAccount) {
	wc := wechat.NewWechat()
	memory := cache.NewMemory()
	conf := &config.Config{
		AppID:          utils.DW_WX_APPID,
		AppSecret:      utils.DW_WX_APP_SECRET,
		Token:          "",
		EncodingAESKey: "",
		Cache:          memory,
	}
	officialAccount = wc.GetOfficialAccount(conf)
	wechatAccessToken := &WechatAccessToken{}
	officialAccount.SetAccessTokenHandle(wechatAccessToken)
	return
}

// GetAccessToken 获取accessToken
func (wechat WechatAccessToken) GetAccessToken() (accessToken string, err error) {
	wxToken, err := models.GetWxTokenById()
	if err != nil {
		return
	}
	//如果300s就要过期了，那么就去刷新accessToken
	if wxToken.ExpiresIn < time.Now().Unix()+300 {
		tmpAccessToken, expires, tmpErr := getTokenFromServer(WxAppId, WxAppSecret)
		if tmpErr != nil {
			err = tmpErr
			return
		}

		var updateCols = []string{"access_token", "expires_in"}
		wxToken.AccessToken = tmpAccessToken
		wxToken.ExpiresIn = expires - 600 //快过期前10分钟就刷新掉
		wxToken.Update(updateCols)
	}
	accessToken = wxToken.AccessToken
	return
}

// getTokenFromServer 服务端获取accessToken
func getTokenFromServer(appid, wxSecret string) (accessToken string, expires int64, err error) {
	apiUrl := "https://api.weixin.qq.com/cgi-bin/token?grant_type=client_credential&appid=%s&secret=%s"
	resAccessToken, err := credential.GetTokenFromServer(fmt.Sprintf(apiUrl, appid, wxSecret))
	if err != nil {
		return
	}

	expires = resAccessToken.ExpiresIn
	accessToken = resAccessToken.AccessToken
	return
}

// GetUserInfo 获取微信用户详情
func GetUserInfo(openid string) (userInfo *user.Info, err error) {
	wechatClient := GetWxChat()
	userClient := wechatClient.GetUser()
	userInfo, err = userClient.GetUserInfo(openid)
	return
}

// GetJsConfig 获取公众号jsConfig
func GetJsConfig(signUrl string) (jsConf *js.Config, err error) {
	wechatClient := GetWxChat()
	j := wechatClient.GetJs()
	jsConf, err = j.GetConfig(signUrl)
	return
}
